@@ -14,11 +14,11 @@ PARLER_LANGUAGES = getattr(settings, 'PARLER_LANGUAGES', {})
 
 PARLER_ENABLE_CACHING = getattr(settings, 'PARLER_ENABLE_CACHING', True)
 
+# Have to fill the default section explicitly to avoid circular imports
+PARLER_LANGUAGES.setdefault('default', {})
+PARLER_LANGUAGES['default'].setdefault('code', PARLER_DEFAULT_LANGUAGE_CODE)
+PARLER_LANGUAGES['default'].setdefault('fallback', PARLER_DEFAULT_LANGUAGE_CODE)
 
 # Cleanup settings
 PARLER_DEFAULT_LANGUAGE_CODE = normalize_language_code(PARLER_DEFAULT_LANGUAGE_CODE)
-PARLER_LANGUAGES = add_default_language_settings(
-    PARLER_LANGUAGES,
-    code=PARLER_DEFAULT_LANGUAGE_CODE,
-    fallback=PARLER_DEFAULT_LANGUAGE_CODE
-)
+PARLER_LANGUAGES = add_default_language_settings(PARLER_LANGUAGES)
